import subprocess
from pathlib import Path


def find_zit_root(start_path: Path):
    current_path = start_path
    while current_path != Path("/"):
        if (current_path / ".zit").is_dir():
            return current_path
        current_path = current_path.parent
    raise FileNotFoundError(
        "No .zit directory found in the current directory or any parent directories, are you inside a zit project?"
    )


class SubprocessError(Exception):
    def __init__(self, returncode, stderr, cmd):
        self.returncode = returncode
        self.stderr = stderr
        self.cmd = cmd
        super().__init__(f"Error running command: {' '.join(cmd)}\n{stderr}")


def run_subprocess(command, cwd=None, capture_stdout=True, shell=False, env=None, wait=True):
    process = subprocess.Popen(
        command,
        stdout=None if capture_stdout else subprocess.DEVNULL,
        stderr=subprocess.PIPE,
        text=True,
        cwd=cwd,
        shell=shell,
        env=env,
    )

    if not wait:
        return process

    _, stderr = process.communicate()

    if process.returncode:
        raise SubprocessError(process.returncode, stderr, command)

    return process



from datetime import datetime

from fastapi import APIRouter, HTTPException

from ...dataset.manager import Manager


def build_categories_router(params, manager: Manager):
    categories_router = r = APIRouter(tags=["Dataset: Categories"])

    @r.patch("/category", summary="Rename a category")
    async def rename_category_r(
        category: str,
        rename_to: str,
        at: str = datetime.now().replace(microsecond=0).isoformat(),
    ):
        anno = manager.anno_df
        categories = anno["category"].unique()

        if category not in categories:
            raise HTTPException(status_code=404, detail=f"Category {category} not found")

        manager.rename_category(category, rename_to, at)
        anno = manager.anno_df

        return anno["category"].unique().tolist()

    return categories_router



import json
import os
from hashlib import blake2s
from pathlib import Path

from ..routes import build_default_router
from ..utils import find_zit_root
from .manager import Manager


def abspath(path):
    return os.path.abspath(path)


class DatasetBuilder:
    def __init__(self, task):
        self.task = task
        self._datasets = {}

    def __call__(self, **kwargs):
        raise NotImplementedError

    @staticmethod
    def _hash_params(params, digest_size=4):
        hasher = blake2s(digest_size=digest_size)
        hasher.update(json.dumps(params, sort_keys=True).encode("utf-8"))
        return hasher.hexdigest()


class DatasetBuilderTemplate(DatasetBuilder):
    def __call__(self, **kwargs):
        zit_root = find_zit_root(Path.cwd())

        image_source = abspath(kwargs.get("image_source", str(zit_root / "images")))
        annotation_source = abspath(kwargs.get("annotation_source", str(zit_root / "annotations.csv")))

        params = {
            "task": self.task,
            "image_source": image_source,
            "annotation_source": annotation_source,
        }

        hash_ = self._hash_params(params)

        if hash_ in self._datasets:
            dataset = self._datasets[hash_]

        else:
            manage_root = str(zit_root / ".zit" / "manager" / hash_)
            os.makedirs(manage_root, exist_ok=True)
            manager = Manager(manage_root, self.task, dataframe_serving=True)
            manager.add_metas(image_source, for_init=True)
            manager.add_annotations(annotation_source, for_init=True)

            meta_serving = kwargs.get("meta_serving", False)
            router, df, meta_df = build_default_router(
                params,
                manager,
                meta_serving=meta_serving,
            )

            dataset = {
                "params": params,
                "manager": manager,
                "router": router,
                "df": df,
                "meta_df": meta_df,
            }

            self._datasets[hash_] = dataset

        return dataset
